/*
   dbfgen.c - produces Clipper code to generate a database with the
   same structure as that specified by the user. Adapted from 
   "dbfstru" in Rick Spence's "Clipper Programming Guide".
  
   jhp 9/14/90

	 Compiles with Turbo C or Microsoft C in small data model.
	 To compile, use either tcc dbfgen.c or cl dbfgen.c.

	 J. Poplett. Email: BIX: jpoplett; internet: john@cstreet.com
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef lint
static char rcsid[] = "$Id: dbfgen.c 1.7 92/01/22 23:52:45 john Exp $";
#endif

typedef struct
{
	char dbf_id;
	char last_update[3];
	long last_rec;
	unsigned data_offset;
	unsigned rec_size;
	char filler[20];
} DBF_HEAD;

typedef struct
{
	char field_name[11];
	char field_type;
	char dummy[4];
	union {
		unsigned char_len;
		struct {
			char len;
			char dec;
		} num_size;
	} len_info;
	char filler[14];
} FIELD_REC;

typedef enum { FALSE = 0, TRUE = 1 } boolean;

DBF_HEAD dbf_head;

FIELD_REC field_rec;

boolean bSummer87 = FALSE;

void specify_record(char *name, char type, int len, int dec)
{
	if (bSummer87)
	{
		(void) printf("\tAPPEND BLANK\n");
		(void) printf("\tREPLACE field_name WITH \"%s\", ", name);
		(void) printf("field_type WITH \"%c\", ;\n", type);
		(void) printf("\t\tfield_len WITH %d, field_dec WITH %d\n", 
			len, dec);
		(void) printf("\n");
	}
	else
	{
		(void) printf("\tAADD(aDBFStruct, { \"%s\", \"%c\", %d, %d})\n", 
			name, type, len, dec);
	}
}

void usage(void)
{
	(void) fprintf(stderr, "usage: dbfgen [-87] filename\n");
	exit(1);
}

void main(int argc, char **argv)
{
	char *dbn, *fn, *p, *option;
	FILE *fp;

	if (argc < 2) 
		usage();

	while (**(argv + 1) == '-') {
		option = *(argv + 1), ++argv;
		if (strcmp(++option, "87") == 0)
			bSummer87 = TRUE;
		else
			usage();
	}

	fn = argv[1];

	if (strrchr(fn, '.') == NULL) 
		fn = strcat(strcpy((char *)malloc(strlen(fn) + 5), fn), ".dbf");

	dbn = strdup((p = strrchr(fn, '\\')) != NULL ? p + 1 : (p = strrchr(fn, '/')) != NULL ? p + 1 : fn);

	if ((p = strrchr(dbn, '.')) != NULL) 
		*p = '\0';

	if ((fp = fopen(fn, "rb")) == NULL) 
	{
		(void) fprintf(stderr, "dbfgen: cannot open %s for reading.\n", fn);
		exit(1);
	}

	if (fread(&dbf_head, sizeof(dbf_head), 1, fp) != 1) 
	{
		(void) fprintf(stderr, "dbfgen: error reading %s.\n", fn);
		exit(1);
	}


	if (bSummer87)
	{
		(void) printf("*\n* PROCEDURE mk_%s_dbf\n*\n\n", dbn);
		(void) printf("PROCEDURE mk_%s_dbf\n", dbn);

		(void) printf("\n");
		(void) printf("\tPARAMETER name\n\n");
		(void) printf("\tPRIVATE save_area\n");
		(void) printf("\tIF PCOUNT() < 1\n");
		(void) printf("\t\tname = \"%s\"\n", dbn);
		(void) printf("\tENDIF\n\n");
		(void) printf("\tsave_area = SELECT()\n\n");
		(void) printf("\tSELECT 0\n");
		(void) printf("\tCREATE dummy\n");
		(void) printf("\n");
	}
	else 
	{
		(void) printf("/*\n  PROCEDURE mk_%s_dbf\n */\n\n", dbn);
		(void) printf("PROCEDURE mk_%s_dbf (cName)\n", dbn);

		(void) printf("\n");
		(void) printf("\tLOCAL aDBFStruct := {}\n\n");
		(void) printf("\tIF cName == Nil\n");
		(void) printf("\t\tcName := \"%s\"\n", dbn);
		(void) printf("\tENDIF\n\n");
	}

	while (fread(&field_rec, sizeof(FIELD_REC), 1, fp) == 1 && field_rec.field_name[0] != 0x0d) {
		switch(field_rec.field_type) {
			case 'D':
				specify_record(field_rec.field_name, field_rec.field_type, 8, 0);
				break;
			case 'N':
				specify_record(field_rec.field_name, field_rec.field_type, 
					field_rec.len_info.num_size.len, 
						field_rec.len_info.num_size.dec);
				break;
			default:
				specify_record(field_rec.field_name, field_rec.field_type, 
					field_rec.len_info.char_len, 0);
				break;
		}
	}

	if (bSummer87)
	{
		(void) printf("\tCREATE (name) FROM dummy\n\n");
		(void) printf("\tUSE\n\n");
		(void) printf("\tSELECT (save_area)\n\n");
	}
	else
	{
		(void) printf("\n\tDBCREATE (cName, aDBFStruct)\n\n");
	}

	(void) printf("RETURN\n");

	(void) fclose(fp);
	exit(0);
}
